<?php
namespace Teknisa\EST\Modules\EstoqueCego\Controllers;

use Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use \Teknisa\Libs\Util\DateUtil;
use \Geral\Entities\Paramfilial;

use Zeedhi\Framework\Report\ReportService;

class Est95100AnaliseInventarioCegoController extends TeknisaCrud {

    protected $dataSourceName = "";
    protected $dataSourceManager;
    protected $environment;
    protected $crudService;
    protected $reportService;
    
    protected $nrorg;
    protected $currentTime;
    protected $operador;
    protected $filialLogada;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, ReportService $reportService, $validacoesEstoqueService) {
        $this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->reportService       = $reportService;
        
        $this->operador = $environment->getCdOperador();
        $this->nrorg = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->currentTime = $environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->environment = $environment;
    }
    
    public function generateReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $reportName = $row['NMRELATORIO'];
        $reportStrategy = $this->getReportStrategyReport($reportName);
        $parametersRow = array();
        foreach ($reportStrategy['parameterMapping'] as $fieldName => $paramName) {
            switch($fieldName){
                case"CDFILIAL":
                    $filialFormatString = "_";
                    foreach($row['CDFILIAL'] as $filial)
                        $filialFormatString = $filialFormatString . $filial . "_";
                        
                    $parametersRow[$paramName] = $filialFormatString;
                    break;
                case"CDALMOXARIFE":
                case"CDLOCALESTOQ":
                    $parametersRow[$paramName] = $row[$fieldName] ?? ' ';
                    if($parametersRow[$paramName] == '')
                        $parametersRow[$paramName] = ' ';
                    break;
                case"CDARVPRODINI":
                case"CDARVPRODFIN":
                    $parametersRow[$paramName] = $fieldName == 'CDARVPRODINI' ? $row['CDARVPRODFIN'] : str_pad($row['CDARVPRODFIN'],  20, "z");
                    break;
                case"DTINICIAL":
                    $parametersRow[$paramName] = $row['DTCONTAGEM']['start'];
                    break;
                case"DTFINAL":
                    $parametersRow[$paramName] = $row['DTCONTAGEM']['end'];
                    break;
                case"IDSITUACEITEINV":
                    $parametersRow[$paramName] = $row['IDSITUACEITEINV'];
                    break;
                case"CDOPERADOR":
                    $parametersRow[$paramName] = $this->operador;
                    break;
                case"UTILALMOX":
                    $parametersRow[$paramName] = $parametersRow['CDALMOXARIFE']  == ' ' && $parametersRow['CDLOCALESTOQ']  == ' ' ? 'N' : 'S' ;
                    break;
                case"ORDENACAO":
                    $parametersRow[$paramName] = $row['ORDENACAO'];
                    break;
                case"QTD":
                    $parametersRow[$paramName] = -1;
                    break;
                case"NRTPARVPROD":
                case"DTCONTAGEM":
                case"FILIALITEN":
                    $parametersRow[$paramName] = '';
                    break;
                case"NVTOT":
                    $parametersRow[$paramName] = $this->getNvTot($row->get($fieldName));
                    break;
                case"CDGRUPPROD":
                    $parametersRow[$paramName] = $row['CDGRUPPROD'];
                    break;
                case"IDGERITINVCEGPR":
                    $parametersRow[$paramName] = $row['IDGERITINVCEGPR'];
                    break;
                default;
                break;
            }
        }
        
        if($parametersRow['CDALMOXARIFE'] == ' ' && $parametersRow['CDLOCALESTOQ'] == ' '){
            $parametersRow['CDALMOXARIFE'] = 'T';
            $parametersRow['CDLOCALESTOQ'] = 'T';
        }
        
        $parametersRow['__format'] = $row["__FORMAT"];
        $birtURL = $this->reportService->createRemoteReport('BIRT', $reportName, $parametersRow);
        $response->addMessage(new DTO\Response\Message($birtURL));
    }
    
    private function getReportStrategyReport($reportName) {
        $reportStrategy = array (
            'strategy' => 'BIRT'
        );
        switch($reportName) {
            case 'EST951001':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "CDFILIAL",
                    "CDALMOXARIFE" => "CDALMOXARIFE",
                    "CDLOCALESTOQ" => "CDLOCALESTOQ",
                    "CDARVPRODINI" => "PRODINI",
                    "CDARVPRODFIN" => "PRODFIN",
                    "DTINICIAL" => "DTINICIAL",
                    "DTFINAL" => "DTFINAL",
                    "IDSITUACEITEINV" => "IDSITUACEITEINV",
                    "CDOPERADOR" => "CDOPERADOR",
                    "ORDENACAO" => "ORDENACAO",
                    "QTD" => "QTD",
                    "NRTPARVPROD" => "NRTPARVPROD",
                    "NVTOT" => "NVTOT"
                );
                break;
            case 'EST951002':
                $reportStrategy['parameterMapping'] = array (
                    "CDFILIAL" => "CDFILIAL",
                    "DTINICIAL" => "DTINICIAL",
                    "DTFINAL" => "DTFINAL",
                    "IDSITUACEITEINV" => "IDSITUACEITEINV",
                    "CDOPERADOR" => "CDOPERADOR",
                    "QTD" => "QTD",
                    "CDGRUPPROD" => "CDGRUPPROD",
                    "CDARVPRODINI" => "PRODINI",
                    "CDARVPRODFIN" => "PRODFIN",
                    "IDGERITINVCEGPR" => "PERCDIV",
                    "DTCONTAGEM" => "DTCONTAGEM",
                    "FILIALITEN" => "FILIALITEN"
                );
                break;
        }
        return $reportStrategy;
    }

    private function getNvTot($nivelEscolhido) {
        $nivelEscolhido  = $nivelEscolhido - 1;
        $filterCriteria  = new FilterCriteria('paramgeral',array());
        $dataSet         = $this->dataSourceManager->findBy($filterCriteria);
        $mascaraProdutos = $dataSet->getRows()[0]['CDPICTPROD'];
        $arrayMascara    = explode(".", $mascaraProdutos);
        for($count = 0, $nvTot = 0; $count <= $nivelEscolhido; $count++) {
            $nvTot += strlen($arrayMascara[$count]);
        }
        return $nvTot;
    }
}
