var Storageable = function Storageable(ZhLogger, requestDataSourceEngine, requestFactory, requestEngine, templateManager, DataSourceHolder, ZHPromise,
	ZhUtil, Query) {
	var self = this;

	var dataSources = DataSourceHolder.dataSources;
	var lastSavePromise = ZHPromise.when();
	var ALL_RECORDS = 999999999;
	var _getDataSourceModel = function(name) {
		var clearedName = DataSourceHolder.clearName(name);
		if (!DataSourceHolder.dataSources[clearedName]) {
			return DataSourceHolder.getDataSource(name).then(null, ZHPromise.when);
		}
		return ZHPromise.when(DataSourceHolder.dataSources[clearedName]);
	};

	self.getDataSource = function() {
		var name = arguments[0];
		var callBack = arguments[1];
		var filters = arguments[2];
		var page = arguments[3];
		var itemsPerPage = arguments[4];
		var deferred = ZHPromise.defer();
		var dataToReturn;
		if (callBack && typeof(callBack) !== 'function') {
			filters = arguments[1];
			page = arguments[2];
			itemsPerPage = arguments[3];
			callBack = false;
		}
		if (!page || page < 0) {
			page = 1;
		}
		if (!itemsPerPage || itemsPerPage < 0) {
			itemsPerPage = 500;
		}

		_getDataSourceModel(name).then(function() {
			self.getDataSourceItems(name)
			.then(function(data) {
				if (data && data !== 'undefined') {
					var orderByFilters = self.extractFiltersByOperator(filters, Query.ORDER);
					var groupByFilters = self.extractFiltersByOperator(filters, Query.GROUP);
					if (groupByFilters && groupByFilters.length) {
						ZhLogger.warning("LocalStorageEngine doesn't support group by.");
					}
					var preparedData = self._prepareData(data);
					dataToReturn = preparedData.filter(function(item) {
						return self.applyFilters(item, filters);
					});
					dataToReturn = self.applyOrder(dataToReturn, orderByFilters);
					dataToReturn = dataToReturn.slice((page - 1) * itemsPerPage, page * itemsPerPage);
				} else {
					dataToReturn = [];
				}
				if (callBack) {
					callBack(dataToReturn);
				}
				deferred.resolve(dataToReturn);
			}).then(undefined, function(error) {
				deferred.reject(error);
			});
		});

		return deferred;
	};


	self.setDataSource = function(name, dataSource, callBack) {
		var deferred = ZHPromise.defer();
		dataSource = angular.isArray(dataSource) ? dataSource : [dataSource];
		if (dataSource) {
			var dataToReturn = dataSource.map(ZhUtil.clearRow);
			self._setItem(name, dataToReturn).then(function() {
				if (callBack) {
					callBack(dataToReturn);
				}
				deferred.resolve(dataToReturn);
			});
		}
		return deferred;
	};

	var DIFF_COMPARATOR = function(objectOne, another, name) {
		var clearedName = DataSourceHolder.clearName(name);
		if (!dataSources[clearedName]) {
			return true;
		}
		var dataSource = dataSources[clearedName][clearedName];
		dataSource.localKey = Array.isArray(dataSource.localKey) ? dataSource.localKey : [dataSource.localKey];
		return dataSource.localKey.some(function(localKey) {
			return objectOne[localKey] !== another[localKey];
		});
	};

	var isFilter = function(filter) {
		return filter && filter.operator && filter.name;
	};

	var _removeWithFilters = function(name, filters) {
		return self.getDataSource(name, [], 1, ALL_RECORDS).then(function(data) {
			data = data.filter(function(item) {
				return !self.applyFilters(item, filters);
			});
			return data;
		});
	};

	var _removeWithDataSource = function(name, dataToRemove) {
		return self.getDataSource(name, [], 1, ALL_RECORDS).then(function(datasource) {
			return datasource.filter(function(item) {
				return dataToRemove.every(function(itemToRemove) {
					return DIFF_COMPARATOR(item, itemToRemove, name);
				});
			});
		});
	};

	self.remove = function() {
		var deferred = ZHPromise.defer();

		var name = arguments[0];
		var filtersOrDatasource;
		var callback;

		if (Util.isFunction(arguments[1])) {
			callback = arguments[1];
		} else {
			filtersOrDatasource = arguments[1];
			callback = arguments[2];
		}

		if (!filtersOrDatasource || (Array.isArray(filtersOrDatasource) && !filtersOrDatasource.length)) {
			self._removeAll(name).then(resolve, deferred.reject);
		} else {
			filtersOrDatasource = Util.parseToArray(filtersOrDatasource);
			if (isFilter(filtersOrDatasource[0])) {
				_removeWithFilters(name, filtersOrDatasource).then(setDataSource, deferred.reject);
			} else {
				_removeWithDataSource(name, filtersOrDatasource).then(setDataSource, deferred.reject);
			}
		}

		function setDataSource(data) {
			self.setDataSource(name, data).then(resolve);
		}

		function resolve() {
			if (callback) {
				callback();
			}
			deferred.resolve();
		}

		return deferred.promise;
	};

	self.save = function(name, object) {
		var resolve = function(source) {
			var dataToSave = angular.isArray(object) ? object : [object];

			var filtered = source.filter(function(sourceItem) {
				return dataToSave.every(function(newData) {
					return DIFF_COMPARATOR(sourceItem, newData, name);
				});
			});
			dataToSave = dataToSave.concat(filtered).map(ZhUtil.clearRow);

			return self.setDataSource(name, dataToSave);
		};
		lastSavePromise = lastSavePromise
			.then(self.getDataSource.bind(self, name, [], 1, ALL_RECORDS))
			.then(resolve);
		return lastSavePromise;
	};

	var _checkAndSave = function(dataSourceName, serverData, comparator, callBack) {
		var deferred = ZHPromise.defer();
		var reject = function() {
			deferred.reject();
		};
		serverData = Array.isArray(serverData) ? serverData : [serverData];
		DataSourceHolder.getDataSource(dataSourceName).then(function() {
			return self.getDataSource(dataSourceName, [], 1, ALL_RECORDS);
		}, reject).then(function(source) {
			var filtered = source.filter(function(sourceItem) {
				return serverData.every(function(newData) {
					return comparator(sourceItem, newData, dataSourceName);
				});
			});
			var result = serverData.concat(filtered);

			return self.setDataSource(dataSourceName, result, function() {
				if (callBack) {
					callBack(serverData, dataSourceName);
				}
			});
		}, reject).then(function() {
			deferred.resolve(serverData);
		}, reject);
		return deferred;
	};

	var _addPackageToDataSourceName = function (pckg, dataSourceName) {
		if (dataSourceName.indexOf("#") === -1) {
			if (dataSourceName.indexOf("/") !== 0) {
				dataSourceName = pckg ? pckg + "#/" + dataSourceName : "/" + dataSourceName;
			} else if (pckg) {
				dataSourceName = pckg + "#" + dataSourceName;
			}
		} else {
			if (dataSourceName.indexOf("#/") === -1) {
				dataSourceName = dataSourceName.split("#").join("#/");
			}
		}
		return dataSourceName;
	};

	var _buildDownloadCallback = function(callBack, comparator, name) {
		var pckg = Util.getPackage(name);
		return function() {
			var data = arguments[0];
			var keys = null;
			var dataSourceName = null;
			if (Array.isArray(arguments[1])) {
				keys = arguments[1];
			} else {
				dataSourceName = arguments[1];
			}
			data = data || [];
			if (!Array.isArray(data) && typeof(data) !== 'object') {
				ZhLogger.error("ZEEDHI ERROR!\nA datasource should be an Array: " + dataSourceName);
			}
			if (dataSourceName) {
				_checkAndSave(_addPackageToDataSourceName(pckg, dataSourceName), data, comparator, callBack);
			} else {
				var promises = [];
				keys.map(function(dataSourceName) {
					var result = data[dataSourceName];
					promises.push(_checkAndSave(_addPackageToDataSourceName(pckg, dataSourceName), result, comparator));
				});
				ZHPromise.all(promises).then(function() {
					callBack(data, keys);
				});
			}
		};
	};

	this.download = function() {
		var name = arguments[0];
		var callBack = arguments[1];
		var filter = arguments[2];
		var comparator = arguments[3] || DIFF_COMPARATOR;
		var retryCount = arguments[4] || 1;
		var timeout = arguments[5] || 15000;
		var retryCallback = arguments[6];
		var deferred;
		var errorCallBack;
		if (!callBack || typeof(callBack) !== 'function') {
			deferred = ZHPromise.defer();
			filter = arguments[1];
			comparator = arguments[2] || DIFF_COMPARATOR;
			retryCount = arguments[3] || 1;
			timeout = arguments[4] || 15000;
			retryCallback = arguments[5];
			callBack = function(data) {
				deferred.resolve(data);
			};
			errorCallBack = function(err) {
				deferred.reject(err);
			};
		}
		callBack = _buildDownloadCallback(callBack, comparator, name);

		_getDataSourceModel(name).then(function() {
			requestDataSourceEngine.getDataSource(name, callBack, filter, 0, 100000, errorCallBack, retryCount, timeout, retryCallback);
		});

		if (deferred) {
			return deferred.promise;
		}
	};

	this.removeFromServer = function(name, callBack, row, retryCount, timeout, retryCallback) {
		return requestDataSourceEngine.remove(name, row, callBack, retryCount, timeout, retryCallback);
	};

	this.saveInServer = function(name, callBack, dataSource, retryCount, timeout, retryCallback) {
		if (dataSource) {
			return requestDataSourceEngine.setDataSource(name, dataSource.map(ZhUtil.clearRow), callBack, retryCount, timeout, retryCallback);
		} else {
			return this.getDataSource(name, [], 1, ALL_RECORDS).then(function(data) {
				return requestDataSourceEngine.setDataSource(name, data.map(ZhUtil.clearRow), callBack, retryCount, timeout, retryCallback);
			});
		}
	};

	this.sync = function(name, callBack, filters) {
		return this.getDataSource(name, filters, 1, ALL_RECORDS).then(function(data) {
			var syncEngine = new SyncEngine(requestFactory, self, requestEngine, templateManager);
			syncEngine.sync(name, data, filters);
			return self.download(name, filters).then(function(serverData) {
				if (Util.isFunction(callBack)) {
					callBack(serverData);
				}
				return serverData;
			});
		});
	};
};